use mongod::{
    assert_reference_of,
    derive::{Model, Referencable},
    Model, Referencable, Reference, Validate,
};
use serde::{Deserialize, Serialize};

use crate::library::artist::Artist;

#[derive(Debug, Clone, Serialize, Deserialize, Model, Referencable)]
pub struct Album {
    pub _id: String,
    pub title: String,
    pub artist_id: Option<Reference>,
}

impl Album {
    pub async fn create(title: &str, artist: Option<&str>) -> Self {
        let a = Self {
            _id: uuid::Uuid::new_v4().to_string(),
            title: title.to_string(),
            artist_id: if let Some(artist) = artist {
                Some(Reference::new_raw(artist).await.unwrap())
            } else {
                None
            },
        };
        a.insert().await.unwrap();
        a
    }
}

impl Validate for Album {
    async fn validate(&self) -> Result<(), String> {
        if let Some(artist_id) = &self.artist_id {
            assert_reference_of!(artist_id, Artist);
        }

        Ok(())
    }
}
